/*
 * Copyright (c) 1992-2002 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/4.2/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.27 $";
#endif

/*
 * Definetion functions
 */
#include "defs.h"
#include "bjhashtab.h"

/*
 * DefineHashGroups contains all Group hash tables.  The key for
 * DefineHashGroups is the Group name.
 */
static htab_t		       *DefinesHashGroups;
/*
 * DefineHashByKeyStr contains all Define_t members with KeyStr as the
 * key.
 */
static htab_t		       *DefinesHashByKeyStr;
/*
 * DefineHashByKeyNum contains all Define_t members with KeyNum as the
 * key.
 */
static htab_t		       *DefinesHashByKeyNum;

/*
 * Valid list of definetions
 * XXX 	It would be nice if only those names that are valid for
 * 	the OS we're run on are defined.
 */
static char *ValidDefineList[] = {
    DL_SYSMODEL, DL_KARCH, DL_CPU, DL_OBP, DL_VPD, 
    DL_NETTYPE, DL_CATEGORY, DL_PART, DL_TAPEINFO,
    DL_KERNEL, DL_SYSCONF, DL_SCSI_DTYPE, DL_CDSPEED,
    NULL
};

/*
 * Make sure Name is valid.
 */
int DefValid(Name)
    char		       *Name;
{
    register char	      **cpp;
    int				Valid = FALSE;

    for (cpp = ValidDefineList; !Valid && cpp && *cpp; ++cpp)
	if (EQ(Name, *cpp))
	    Valid = TRUE;

    return(Valid);
}

/*
 * Dump contents of hash Table for debug
 */
static void DumpHash(Table, Name)
     htab_t		       *Table;
     char		       *Name;
{
    if (!Debug)
	return;

    if (!bjhfirst(Table)) {
	SImsg(SIM_DBG, "bjhfirst for table <%s> failed", Name);
	return;
    }

    do {
	SImsg(SIM_DBG, "HASH TABLE <%s> key=<%.*s>",
	      Name,
	      bjhkeyl(Table), bjhkey(Table));
    } while (bjhnext(Table));
}

/*
 * Create a key combining String1, String2, and Num with Group.
 * This insures that Key is unique enough.
 */
static char *MakeKey(Group, String1, String2, Num)
     char		       *Group;
     char		       *String1;
     char		       *String2;
     long			Num;
{
    static char			Key[1024];
    static char			NumStr[1024];
    static char			String1L[1024];
    static char			String2L[1024];

    if (!Group)
	return (char *) NULL;

    NumStr[0] = String1L[0] = String2L[0] = CNULL;

    if (String1)
	strlower2(String1, String1L, sizeof(String1L));
    if (String2)
	strlower2(String2, String2L, sizeof(String2L));
    if (Num >= 0)
	snprintf(NumStr, sizeof(NumStr), "0x%x", Num);

    /*
     * Group should already be lowercase.
     */
    snprintf(Key, sizeof(Key), "%s", Group);
    if (String1L[0])
	StrAppend(Key, sizeof(Key), String1L);
    if (String2L[0])
	StrAppend(Key, sizeof(Key), String2L);
    if (NumStr[0])
	StrAppend(Key, sizeof(Key), NumStr);

    return Key;
}

/*
 * Add a definetion and associate it with Group
 */
extern void DefAdd(Define, Group)
    Define_t		       *Define;
    char		       *Group;
{
    htab_t		       *GroupTab;
    char		        GroupKey[128];
    char		       *Key;

    if (!Define || !Group || !Group[0])
	return;

    if (!DefinesHashGroups)
	DefinesHashGroups = bjhcreate(128);	/* Arbitrary start size */
    if (!DefinesHashByKeyStr)
	DefinesHashByKeyStr = bjhcreate(512);	/* Arbitrary start size */
    if (!DefinesHashByKeyNum)
	DefinesHashByKeyNum = bjhcreate(512);	/* Arbitrary start size */

    strlower2(Group, GroupKey, sizeof(GroupKey));

    /*
     * Find the master group table to add this entry to.
     */
    if (bjhfind(DefinesHashGroups, GroupKey, strlen(GroupKey))) {
	GroupTab = (htab_t *) bjhstuff(DefinesHashGroups);
    } else {
	/* Create a group table */
	GroupTab = bjhcreate(256);
#if	DEBUG_DEFADD
	SImsg(SIM_DBG, "DefAdd: Adding <%s> to table Groups", GroupKey);
#endif
	/* Add the group table to the master table of tables */
	if (!bjhadd(DefinesHashGroups, strdup(GroupKey), strlen(GroupKey),
		    GroupTab))
	    SImsg(SIM_DBG,
		  "DefAdd: Add <%s> to table of Groups failed: Entry exists?",
		  GroupKey);
    }

    /*
     * Add entry to its group table
     */
    if (Key = MakeKey(GroupKey, Define->KeyStr, Define->ValStr1,
		      Define->KeyNum)) {
#if	DEBUG_DEFADD
	SImsg(SIM_DBG, "DefAdd: Adding <%s> to Group table <%s>",
	      Key, GroupKey);
#endif
	if (!bjhadd(GroupTab, strdup(Key), strlen(Key), Define))
	    SImsg(SIM_DBG,
		  "DefAdd: Add <%s> to Group table <%s> failed: Entry exists?",
		  Key, GroupKey);
    }

    /*
     * Add entry to general table By KeyStr
     */
    if (Define->KeyStr && (Key = MakeKey(GroupKey, Define->KeyStr,
					 NULL, -1))) {
#if	DEBUG_DEFADD
	SImsg(SIM_DBG, "DefAdd: Adding <%s> to table ByKeyStr", Key);
#endif
	if (!bjhadd(DefinesHashByKeyStr, strdup(Key), strlen(Key), Define))
	    SImsg(SIM_DBG,
		  "DefAdd: Add <%s> to table ByKeyStr failed: Entry exists?",
		  Key);

    }

    /*
     * Add entry to general table By KeyNum
     */
    if (Define->KeyNum >= 0 &&
	(Key = MakeKey(GroupKey, NULL, NULL, Define->KeyNum))) {
#if	DEBUG_DEFADD
	SImsg(SIM_DBG, "DefAdd: Adding <%s> to table ByKeyNum", Key);
#endif
	if (!bjhadd(DefinesHashByKeyNum, strdup(Key), strlen(Key), Define))
	    SImsg(SIM_DBG,
		  "DefAdd: Add <%s> to table ByKeyNum failed: Entry exists?",
		  Key);
    }
}

/*
 * Get all Define_t's associated with Group
 */
extern Define_t *DefGetList(Group)
    char		       *Group;
{
    Define_t		       *List = NULL;
    Define_t		       *Last = NULL;
    Define_t		       *DefPtr;
    htab_t		       *GroupTab;
    char			GroupKey[1024];

    strlower2(Group, GroupKey, sizeof(GroupKey));

    if (bjhfind(DefinesHashGroups, GroupKey, strlen(GroupKey)))
	GroupTab = (htab_t *) bjhstuff(DefinesHashGroups);
    else
	/* Group table doesn't exist */
	return (Define_t *) NULL;

    if (!bjhfirst(GroupTab))
	/* Failed to locate position to first entry */
	return (Define_t *) NULL;

    do {
	DefPtr = (Define_t *) bjhstuff(GroupTab);
	DefPtr->Next = NULL;

	if (Last) {
	    Last->Next = DefPtr;
	    Last = DefPtr;
	} else {
	    List = Last = DefPtr;
	}
    } while (bjhnext(GroupTab));

    return List;
}

/*
 * Destroy DefList 
*/
extern int DefDestroy(DefList)
     Define_t		       *DefList;
{
    register Define_t	       *dp;
    register Define_t	       *Last;

    if (!DefList)
	return -1;

    for (dp = DefList; dp; ) {
	if (dp->KeyStr)		(void) free(dp->KeyStr);
	if (dp->ValStr1)	(void) free(dp->ValStr1);
	if (dp->ValStr2)	(void) free(dp->ValStr2);
	if (dp->ValStr3)	(void) free(dp->ValStr3);
	if (dp->ValStr4)	(void) free(dp->ValStr4);
	if (dp->ValStr5)	(void) free(dp->ValStr5);

	Last = dp;
	dp = dp->Next;
	(void) free(Last);
    }

    return 0;
}

/*
 * Find definetion with key KeyStr and/or KeyNum in list Group.
 */
extern Define_t *DefGet(Group, KeyStr, KeyNum, Opts)
    char		       *Group;
    char		       *KeyStr;
    long			KeyNum;
    int				Opts;
{
    Define_t		       *List;
    register Define_t	       *Ptr = NULL;
    int				StrMatch = FALSE;
    int				NumMatch = FALSE;
    char			GroupKey[1024];
    char		       *Key;

    strlower2(Group, GroupKey, sizeof(GroupKey));

    /*
     * If DO_REGEX is not enabled and we only want to find an
     * entry based on one Key, then use fast hash table lookups.
     */
    if (!FLAGS_ON(Opts, DO_REGEX)) {
	if (KeyStr && !KeyNum && (Key = MakeKey(GroupKey, KeyStr, NULL, -1))) {
	    if (bjhfind(DefinesHashByKeyStr, Key, strlen(Key))) {
		Ptr = (Define_t *) bjhstuff(DefinesHashByKeyStr);
		return Ptr;
	    }
	}
	if (KeyNum >= 0 && !KeyStr &&
	    (Key = MakeKey(GroupKey, NULL, NULL, KeyNum))) {
	    if (bjhfind(DefinesHashByKeyNum, Key, strlen(Key))) {
		Ptr = (Define_t *) bjhstuff(DefinesHashByKeyNum);
		return Ptr;
	    }
	}
    }

    SImsg(SIM_DBG,
	  "DefGet: Performing slow lookup: Group=<%s> GroupKey=<%s> KeyStr=<%s> KeyNum=0x%lx Opts=0x%x",
	  Group, GroupKey, PRTS(KeyStr), KeyNum, Opts);

    /*
     * We couldn't find the requested entry via fast hash table lookup
     * so we'll need to locate the Group list and iterate through that
     * looking for a match.
     */
    List = DefGetList(GroupKey);
    if (!List) {
	SImsg(SIM_DBG, "DefGet: Invalid define group name `%s'.", Group);
	return((Define_t *) NULL);
    }

    for (Ptr = List; Ptr; Ptr = Ptr->Next) {
	if (KeyStr && Ptr->KeyStr)
	    if (FLAGS_ON(Opts, DO_REGEX)) {
		strtolower(KeyStr);
		strtolower(Ptr->KeyStr);
		if (REmatch(KeyStr, Ptr->KeyStr, NULL) > 0)
		    StrMatch = TRUE;
	    } else {
		if (EQ(KeyStr, Ptr->KeyStr))
		    StrMatch = TRUE;
	    }
	/*
	 * If KeyStr is NULL and the entry key is "-" this implies
	 * that we want to match againt none (NULL).
	 */
	if (!StrMatch)
	    if (!KeyStr && Ptr->KeyStr && EQ(Ptr->KeyStr, "-"))
		StrMatch = TRUE;
	if ((KeyNum >= 0) && (Ptr->KeyNum >= 0) && KeyNum == Ptr->KeyNum)
	    NumMatch = TRUE;
	if (KeyStr && (KeyNum >= 0) && StrMatch && NumMatch)
	    return(Ptr);
	if (KeyStr && StrMatch)
	    return(Ptr);
	if ((KeyNum >= 0) && NumMatch)
	    return(Ptr);
    }

    return((Define_t *) NULL);
}
