/*
 * Copyright (c) 1992-2002 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/4.2/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.230 $";
#endif

/*
 * Device routines
 */

#include <stdio.h>
#include "defs.h"

static void DevTreeVerify();

static DevInfo_t	       *OrphanList = NULL;

/*
 * For AssignDevUnits
 */
struct DevUnits {
    char		       *Name;
    int				LastUnit;
    struct DevUnits	       *Next;
};
static struct DevUnits	       *DevUnits;

/*
 * Initialize a ProbeData_t type
 */
extern int ProbeDataInit(Data)
     ProbeData_t	       *Data;
{
    if (!Data)
	return -1;

    memset(Data, CNULL, sizeof(ProbeData_t));

    return 0;
}

/*
 * Initialize DevData_t type
 */
extern DevDataInit(Data)
     DevData_t		       *Data;
{
    if (!Data)
	return -1;

    memset(Data, CNULL, sizeof(DevData_t));
    Data->DevUnit = Data->DevType = Data->Slave = -1;

    return 0;
}

/*
 * Get the prime description of a device.
 */
extern char *PrimeDesc(DevInfo)
    DevInfo_t		       *DevInfo;
{
    register DevDesc_t	       *DevDesc;

    if (!DevInfo)
	return((char *) NULL);

    for (DevDesc = DevInfo->DescList; DevDesc; DevDesc = DevDesc->Next)
	if (FLAGS_ON(DevDesc->Flags, DA_PRIME))
	    return(DevDesc->Desc);

    return((char *) NULL);
}

/*
 * Add a device description to a device.
 */
extern int AddDevDesc(DevInfo, Desc, Label, Action)
    DevInfo_t		       *DevInfo;
    char		       *Desc;
    char		       *Label;
    int				Action;
{
    return AddDesc(&(DevInfo->DescList), Label, Desc, Action);
}

/*
 * Add Resolution to the list of Resolutions in DevInfo.
 * Resolutions is assumed to already be a permament buffer.
 */
extern void DevAddRes(DevInfo, Resolution)
     DevInfo_t		       *DevInfo;
     char		       *Resolution;
{
    register int		i;
    Monitor_t		       *Mon = NULL;

    if (!DevInfo || !Resolution)
	return;

    if (!DevInfo->DevSpec) {
	Mon = (Monitor_t *) xcalloc(1, sizeof(Monitor_t *));
	DevInfo->DevSpec = (void *) Mon;
    } else
	Mon = (Monitor_t *) DevInfo->DevSpec;

    if (!Mon->Resolutions) {
	Mon->Resolutions = (char **) xcalloc(2, sizeof(char *));
	Mon->Resolutions[0] = Resolution;
	return;
    }

    /*
     * Get count of current number of resolutions in list
     * while checking to make sure there is no matching entry.
     */
    for (i = 0; Mon->Resolutions[i]; ++i)
	/* Do case sensitive match check */
	if (eq(Mon->Resolutions[i], Resolution))
	    return;

    /*
     * Re alloc buffer and add File
     */
    Mon->Resolutions = (char **) xrealloc(Mon->Resolutions, 
					  (i+2)*sizeof(char *));
    Mon->Resolutions[i] = strdup(Resolution);
    Mon->Resolutions[i+1] = NULL;
}

/*
 * Add File to the list of Files in DevInfo.
 * File is assumed to already be a permament buffer.
 */
extern void DevAddFile(DevInfo, File)
     DevInfo_t		       *DevInfo;
     char		       *File;
{
    register char	      **cpp;
    register int		i;

    if (!DevInfo || !File)
	return;

    if (!DevInfo->Files) {
	DevInfo->Files = (char **) xcalloc(2, sizeof(char *));
	DevInfo->Files[0] = File;
	return;
    }

    /*
     * Get count of current number of files in list
     * while checking to make sure there is no matching entry.
     */
    for (i = 0, cpp = DevInfo->Files; cpp && cpp[i]; ++i)
	/* Do case sensitive match check */
	if (strcmp(cpp[i], File) == 0)
	    return;

    /*
     * Re alloc buffer and add File
     */
    DevInfo->Files = (char **) xrealloc(DevInfo->Files, (i+1)*sizeof(char *));
    DevInfo->Files[i] = File;
    DevInfo->Files[i+1] = NULL;
}

/*
 * Interface function between mcSysInfo() and underlying device probes.
 */
extern int DeviceInfoMCSI(Query)
     MCSIquery_t	       *Query;
{
    static DevInfo_t 	       *Root;
    int				Status;
    PSI_t		       *PSIptr;

    if (!Query) {
	errno = ENXIO;
	return -1;
    }

    switch (Query->Op) {
    case MCSIOP_CREATE:
	if (!Root) {
	    if (!PsiSwitch.BuildDevices || !PsiSwitch.BuildDevices->Get) {
		SImsg(SIM_DBG, 
		      "Support for `Device' class information is not available on this platform.");
		return 0;
	    }

	    SImsg(SIM_DBG, "BUILDING Device Tree ...");
	    /*
	     * Call each defined BuildDevices function
	     */
	    for (Status = 0, PSIptr = PsiSwitch.BuildDevices;
		 PSIptr && PSIptr->Get; ++PSIptr) {
		Status = (int) (PSIptr->Get)(&Root, Query->SearchExp);
	    }
	    if (Status != 0)
		return -1;
	}

	if (!Root) {
	    SImsg(SIM_DBG, "No devices were found.");
	    errno = ENOENT;
	    return -1;
	}

	DevTreeVerify(&Root);

	Query->Out = (Opaque_t) Root;
	Query->OutSize = sizeof(DevInfo_t *);
	break;
    case MCSIOP_DESTROY:
	if (Query->Out && Query->OutSize)
	    return DestroyDeviceTree((DevInfo_t *) Query->Out);
	break; 
    }

    return 0;
}

/*
 * --RECURSE--
 * Create a new DevInfo_t and optionally copy an old DevInfo_t.
 */
extern DevInfo_t *DevInfoNew(Old)
    DevInfo_t 		       *Old;
{
    register DevInfo_t 	       *SlavePtr, *Slave;
    DevInfo_t 		       *New = NULL;

    New = (DevInfo_t *) xcalloc(1, sizeof(DevInfo_t));

    /* Set int's to -1 */
    New->Type = New->Unit = New->Addr = New->Prio = 
	New->Vec = New->Bus = New->DevNum = New->Port = -1;

    if (!Old)
	return(New);

    /* Bulk copy what we can */
    memcpy((char *) New, (char *) Old, sizeof(DevInfo_t));

    New->Next = NULL;

    /* Copy contents of pointers */
    if (Old->Name)	New->Name = strdup(Old->Name);
    if (Old->Vendor)	New->Vendor = strdup(Old->Vendor);
    if (Old->Serial)	New->Serial = strdup(Old->Serial);
    if (Old->Capacity)	New->Capacity = strdup(Old->Capacity);
    if (Old->Speed)	New->Speed = strdup(Old->Speed);
    if (Old->Revision)	New->Revision = strdup(Old->Revision);
    if (Old->Model)	New->Model = strdup(Old->Model);
    if (Old->ModelDesc) New->ModelDesc = strdup(Old->ModelDesc);
    if (Old->DescList) 	New->DescList = Old->DescList;
    if (Old->Ident)	New->Ident = IdentCreate(Old->Ident);

    /* Copy Slave info */
    for (Slave = Old->Slaves; Slave; Slave = Slave->Next) {
	/* Find last slave */
	for (SlavePtr = New->Slaves; SlavePtr && SlavePtr->Next; 
	     SlavePtr = SlavePtr->Next);
	/* Copy Old slave to last new slave device */
	SlavePtr = NewDevInfo(Slave);
    }

    return(New);
}

/*
 * Backwards compat
 */
extern DevInfo_t *NewDevInfo(Old)
     DevInfo_t		       *Old;
{
    return DevInfoNew(Old);
}

/*
 * Create a new DiskPart_t and optionally copy an old DiskPart_t.
 */
extern DiskPart_t *NewDiskPart(Old)
    DiskPart_t 		       *Old;
{
    DiskPart_t 		       *New = NULL;

    New = (DiskPart_t *) xcalloc(1, sizeof(DiskPart_t));

    if (Old) {
	/* Copy old values to new */
	if (Old->PartInfo)	New->PartInfo = PartInfoCreate(Old->PartInfo);
    }

    return(New);
}

/*
 * Create a new DiskDrive_t and optionally copy an old DiskDrive_t.
 */
extern DiskDrive_t *NewDiskDrive(Old)
    DiskDrive_t 	       *Old;
{
    register DiskPart_t        *dp, *pdp;
    DiskDrive_t 	       *New = NULL;

    New = (DiskDrive_t *) xcalloc(1, sizeof(DiskDrive_t));

    if (!Old)
	return(New);

    /* Bulk copy what we can */
    memcpy((char *) New, (char *) Old, sizeof(DiskDrive_t));

    /* Copy contents of pointers */
    if (Old->Label)	New->Label = strdup(Old->Label);
    if (Old->Ctlr) 	New->Ctlr = NewDevInfo(Old->Ctlr);

    /* Copy partition info */
    for (dp = Old->DiskPart; dp; dp = dp->Next) {
	/* Find last DiskPart_t */
	for (pdp = New->DiskPart; pdp && pdp->Next; pdp = pdp->Next);
	/* Copy old DiskPart_t to last New DiskPart_t */
	pdp = NewDiskPart(dp);
    }

    return(New);
}

/*
 * Create a new DiskDriveData_t and optionally copy an old DiskDriveData_t.
 */
extern DiskDriveData_t *NewDiskDriveData(Old)
    DiskDriveData_t 	       *Old;
{
    DiskDriveData_t 	       *New = NULL;

    New = (DiskDriveData_t *) xcalloc(1, sizeof(DiskDriveData_t));

    if (!Old)
	return(New);

    /* Bulk copy what we can */
    memcpy((char *) New, (char *) Old, sizeof(DiskDriveData_t));

    return(New);
}

/*
 * Create a new FrameBuffer_t and optionally copy an old FrameBuffer_t.
 */
extern FrameBuffer_t *NewFrameBuffer(Old)
    FrameBuffer_t 	       *Old;
{
    FrameBuffer_t 	       *New = NULL;

    New = (FrameBuffer_t *) xcalloc(1, sizeof(FrameBuffer_t));

    if (!Old)
	return(New);

    /* Bulk copy what we can */
    memcpy((char *) New, (char *) Old, sizeof(FrameBuffer_t));

    return(New);
}

/*
 * Create a new Monitor_t and optionally copy an old Monitor_t.
 */
extern Monitor_t *NewMonitor(Old)
    Monitor_t 	       *Old;
{
    Monitor_t 	       *New = NULL;

    New = (Monitor_t *) xcalloc(1, sizeof(Monitor_t));

    if (!Old)
	return(New);

    /* Bulk copy what we can */
    memcpy((char *) New, (char *) Old, sizeof(Monitor_t));

    return(New);
}

/*
 * Create a new NetIF_t and optionally copy an old NetIF_t.
 */
extern NetIF_t *NewNetif(Old)
    NetIF_t 		       *Old;
{
    NetIF_t 		       *New = NULL;

    New = (NetIF_t *) xcalloc(1, sizeof(NetIF_t));

    if (!Old)
	return(New);

    /* Copy */
    New->HostAddr = strdup(Old->HostAddr);
    New->HostName = strdup(Old->HostName);
    New->MACaddr = strdup(Old->MACaddr);
    New->MACname = strdup(Old->MACname);
    New->NetAddr = strdup(Old->NetAddr);
    New->NetName = strdup(Old->NetName);

    return(New);
}

/*
 * --RECURSE--
 * Search for device by address (DevInfo) and return that
 * device's parent.
 */
extern DevInfo_t *FindDeviceParent(DevInfo, TreePtr, Parent)
    DevInfo_t		       *DevInfo;
    DevInfo_t 		       *TreePtr;
    DevInfo_t		       *Parent;
{
    DevInfo_t 		       *Ptr;

    if (DevInfo && DevInfo == TreePtr)
	return(Parent);

    if (TreePtr->Slaves)
	if (Ptr = FindDeviceParent(DevInfo, TreePtr->Slaves, TreePtr))
	    return(Ptr);

    if (TreePtr->Next)
	if (Ptr = FindDeviceParent(DevInfo, TreePtr->Next, TreePtr))
	    return(Ptr);

    return((DevInfo_t *) NULL);
}

/*
 * Check to see if device's dev1 and dev2 are consistant.
 * If there is a discrepancy between the two due to one
 * device not having it's value set, then set it to the
 * other device's value.  Basically this "fills in the blanks".
 */
extern void DevGraft(Dev1, Dev2)
     DevInfo_t 		       *Dev1;
     DevInfo_t 		       *Dev2;
{
#define CHECK(a,b) \
    if (a != b) { \
	if (a) \
	    b = a; \
	else if (b) \
	    a = b; \
    }

    CHECK(Dev1->Type, 		Dev2->Type);
    CHECK(Dev1->ClassType,	Dev2->ClassType);
    CHECK(Dev1->Aliases,	Dev2->Aliases);
    CHECK(Dev1->Vendor,		Dev2->Vendor);
    CHECK(Dev1->Model, 		Dev2->Model);
    CHECK(Dev1->ModelDesc, 	Dev2->ModelDesc);
    CHECK(Dev1->Ident,		Dev2->Ident);
    CHECK(Dev1->Serial,		Dev2->Serial);
    CHECK(Dev1->Capacity,	Dev2->Capacity);
    CHECK(Dev1->Speed,		Dev2->Speed);
    CHECK(Dev1->Revision,	Dev2->Revision);
    CHECK(Dev1->DescList, 	Dev2->DescList);
    CHECK(Dev1->Unit, 		Dev2->Unit);
    CHECK(Dev1->Port, 		Dev2->Port);
    CHECK(Dev1->Bus, 		Dev2->Bus);
    CHECK(Dev1->DevNum,		Dev2->DevNum);
    CHECK(Dev1->Addr, 		Dev2->Addr);
    CHECK(Dev1->Prio, 		Dev2->Prio);
    CHECK(Dev1->Vec, 		Dev2->Vec);
    CHECK(Dev1->DevSpec, 	Dev2->DevSpec);
    CHECK(Dev1->Master, 	Dev2->Master);
    CHECK(Dev1->NodeID, 	Dev2->NodeID);

#undef CHECK
}

/*
 * Is Name in Argv?
 */
extern int SearchCheck(Name, Argv)
    char		       *Name;
    char		      **Argv;
{
    register char	      **cpp;

    for (cpp = Argv; cpp && *cpp; ++cpp) {
	if (EQ(Name, *cpp))
	    return(1);
    }

    return(0);
}

/*
 * Remove DevInfo from the Parent's list of children.
 */
extern int DeviceRemoveLinks(DevInfo, Parent)
     DevInfo_t		       *DevInfo;
     DevInfo_t		       *Parent;
{
    register DevInfo_t	       *Ptr;
    register DevInfo_t	       *Last = NULL;

    if (!DevInfo || !Parent)
	return(-1);

    SImsg(SIM_DBG, "RemoveLinks   DevName=<%s> Parent=<%s>", 
	  PRTS(DevInfo->Name), PRTS(Parent->Name));

    if (Parent->Slaves) {
	for (Ptr = Parent->Slaves; Ptr && Ptr->Next; ) {
	    if (Ptr == DevInfo) {
		if (Last) 
		    /* Delete from previous slave */
		    Last->Next = DevInfo->Next;
		else
		    /* There are no other slaves */
		    Parent->Slaves = NULL;
		DevInfo->Next = NULL;
	    }
	    Last = Ptr;
	    Ptr = Ptr->Next;
	}
    } else
	Parent->Slaves = DevInfo->Next;

    return(0);
}

/*
 * Add DevInfo to a linked list of orphaned devices for later
 * re-attachment.
 */
static void AddOrphanList(DevInfo)
     DevInfo_t		      *DevInfo;
{
    static DevInfo_t	       *Last;

    SImsg(SIM_DBG, "AddOrphanList DevName=<%s>", PRTS(DevInfo->Name));

    /*
     * Make sure we're not being asked to create a loop
     */
    if (Last == DevInfo) {
	SImsg(SIM_DBG, "AddOrphanList: DevName=<%s> Avoiding loop",
	      PRTS(DevInfo->Name));
	return;
    }

    if (Last) {
	Last->Next = DevInfo;
	Last = DevInfo;
    } else {
	Last = OrphanList = DevInfo;
    }
}

/*
 * Check to see if the child (DevInfo) matches the Parent (Master).
 * If not, detach and add to orphan list for later re-attachment
 * with the right Parent.
 */
static int DevTreeCheck(Parent, DevInfo)
     DevInfo_t		      *Parent;
     DevInfo_t		      *DevInfo;
{
    if (!Parent || !DevInfo)
	return(-1);

    if (DevInfo->Master && DevInfo->MasterName && 
	(DevInfo->Master != Parent || 
	 !EQ(DevInfo->MasterName, Parent->Name))) {
	DeviceRemoveLinks(DevInfo, Parent);
	AddOrphanList(DevInfo);
	return(0);
    } else
	return(1);
}

/*
 * -RECURSE-
 * Scan the device tree.
 */
static void DevTreeScan(Parent, DevInfo)
     DevInfo_t		      *Parent;
     DevInfo_t		      *DevInfo;
{
    register DevInfo_t	       *Ptr;

    (void) DevTreeCheck(Parent, DevInfo);

    for (Ptr = DevInfo->Slaves; Ptr; Ptr = Ptr->Next) {
	if (DevTreeCheck(DevInfo, Ptr) == 1)
	    DevTreeScan(DevInfo, Ptr);
    }

    if (DevInfo->Next)
	DevTreeScan(Parent, DevInfo->Next);
}

/*
 * Check the entire device tree (recursively) to see if the
 * Master identified by the device and the Master's list of
 * child devices agree.  Believe the master indicated by the
 * device over the Master's list.
 */
static void DevTreeVerify(TreePtr)
     DevInfo_t		     **TreePtr;
{
    register DevInfo_t	       *Ptr;

    OrphanList = NULL;

    /* Find and detach orphans */
    DevTreeScan(NULL, *TreePtr);

    /* Re-attach each orphan */
    for (Ptr = OrphanList; Ptr; Ptr = Ptr->Next)
	AddDevice(Ptr, TreePtr, NULL);
}

/*
 * Create a device entry.
 * All device Probes should call DeviceCreate() to create and return
 * the base DevInfo from which they can modify as needed.
 */
extern DevInfo_t *DeviceCreate(ProbeData)
     ProbeData_t	       *ProbeData;
{
    DevInfo_t		       *DevInfo;
    char 		       *Name;
    DevData_t 		       *DevData;
    DevDefine_t	 	       *DevDefine;
    register char	      **cpp;
    register int		Count;

    if (!ProbeData)
	return((DevInfo_t *) NULL);

    Name = ProbeData->DevName;
    DevData = ProbeData->DevData;
    DevDefine = ProbeData->DevDefine;

    /*
     * DT_GENERIC devices MUST be marked alive to proceed
     */
    if (DevDefine && DevDefine->Type == DT_GENERIC && 
	!(FLAGS_ON(DevData->Flags, DD_IS_ALIVE) ||
	  FLAGS_ON(DevDefine->Flags, DDT_ZOMBIE) ||
	  FLAGS_ON(DevData->Flags, DD_MAYBE_ALIVE)))
	return((DevInfo_t *) NULL);

    /*
     * Select new DevInfo.  Use UseDevInfo if passed in, or create a new one.
     */
    if (ProbeData->UseDevInfo)
	DevInfo = ProbeData->UseDevInfo;
    else
	DevInfo = NewDevInfo((DevInfo_t *) NULL);

    /*
     * Set our DevInfo (Device) Type
     */
    if (DevInfo->Type <= 0) {
	if (DevData && DevData->DevType)
	    DevInfo->Type = DevData->DevType;
	else if (DevDefine && DevDefine->Type)
	    DevInfo->Type = DevDefine->Type;
    }

    /*
     * Decide on/create our device name (with unit number) i.e. 'vx0'
     */
    if (!DevInfo->Name) {
	if (Name)
	    DevInfo->Name = strdup(Name);
	else if (DevDefine)
	    DevInfo->Name = MkDevName(DevData->DevName, DevData->DevUnit,
				      DevInfo->Type, DevDefine->Flags);
	else if (DevData)
	    DevInfo->Name = MkDevName(DevData->DevName, DevData->DevUnit,
				      DevInfo->Type, 0);
	else 
	    DevInfo->Name = DevData->DevName;
    }

    /*
     * Set what we know from ProbeData
     */
    if (!DevInfo->Master)
	DevInfo->Master = ProbeData->CtlrDevInfo;
    if (ProbeData->DevFile)
	DevAddFile(DevInfo, strdup(ProbeData->DevFile));

    /*
     * Set what we know from DevDefine
     */
    if (DevDefine) {
	DevInfo->Model = DevDefine->Model;
	if (!DevInfo->Vendor)
	    DevInfo->Vendor = DevDefine->Vendor;
	if (DevDefine->Desc)
	    DevInfo->ModelDesc = DevDefine->Desc;
	if (!DevInfo->ClassType)
	    DevInfo->ClassType = DevDefine->ClassType;
    }

    /*
     * Set what we want from DevData
     */
    if (DevData) {
	if (!DevInfo->Master)
	    DevInfo->Master = DevData->CtlrDevInfo;
	if (!DevInfo->Driver)
	    DevInfo->Driver = DevData->DevName;
	if (DevInfo->Unit < 0)
	    DevInfo->Unit = DevData->DevUnit;
	if (DevInfo->NodeID == 0 || DevInfo->NodeID == -1 || 
	    DevInfo->NodeID == -2)
	    DevInfo->NodeID = DevData->NodeID;
	if (!DevInfo->Master)
	    DevInfo->Master = MkMasterFromDevData(DevData);
    }

    /*
     * If we have OS provided info, use it for whatever was not already set
     */
    if (DevData && DevData->OSDevInfo) {
	if (!DevInfo->Vendor)
	    DevInfo->Vendor = DevData->OSDevInfo->Vendor;
	if (!DevInfo->Model)
	    DevInfo->Model = DevData->OSDevInfo->Model;
	if (!DevInfo->ModelDesc)
	    DevInfo->ModelDesc = DevData->OSDevInfo->ModelDesc;
	if (!DevInfo->Ident)
	    DevInfo->Ident = DevData->OSDevInfo->Ident;
	if (!DevInfo->Serial)
	    DevInfo->Serial = DevData->OSDevInfo->Serial;
	if (!DevInfo->Capacity)
	    DevInfo->Capacity = DevData->OSDevInfo->Capacity;
	if (!DevInfo->Speed)
	    DevInfo->Speed = DevData->OSDevInfo->Speed;
	if (!DevInfo->Revision)
	    DevInfo->Revision = DevData->OSDevInfo->Revision;
	if (!DevInfo->DescList)
	    DevInfo->DescList = DevData->OSDevInfo->DescList;
	if (DevInfo->ClassType <= 0)
	    DevInfo->ClassType = DevData->OSDevInfo->ClassType;
	if (!DevInfo->Master)
	    DevInfo->Master = DevData->OSDevInfo->Master;
    }

    if (ProbeData->AliasNames) {
	/*
	 * Copy all aliases that are not the same as the DevInfo Name
	 */
	for (Count = 0, cpp = ProbeData->AliasNames; cpp && *cpp; ++cpp)
	    if (!EQ(*cpp, DevInfo->Name))
		++Count;
	if (Count) {
	    DevInfo->Aliases = (char **) xcalloc(Count+1, sizeof(char *));
	    for (Count = 0, cpp = ProbeData->AliasNames; cpp && *cpp; ++cpp)
		if (!EQ(*cpp, DevInfo->Name))
		    DevInfo->Aliases[Count++] = *cpp;
	}
    }

    return(DevInfo);
}

/*
 * Create a device type based on DevInfo provided from
 * some type of OS source.
 */
extern DevInfo_t *ProbeOSDevInfo(ProbeData)
     ProbeData_t	       *ProbeData;
{
    DevInfo_t		       *DevInfo;
    DevType_t		       *DevType;
    static DevDefine_t		DevDef;
    char		       *DevName;
    DevData_t		       *DevData;

    if (!ProbeData)
	return((DevInfo_t *) NULL);

    DevData = ProbeData->DevData;
    if (!(DevInfo = DevData->OSDevInfo))
	return((DevInfo_t *) NULL);

    if (DevInfo->Name)
	/* Use the provided DevName if set */
	DevName = DevInfo->Name;
    else
	DevName = MkDevName(
		    (DevInfo->Name) ? DevInfo->Name : DevData->DevName, 
		    DevData->DevUnit, -1, 0);

    /*
     * See if we can find a Device Type and if so, call it's 
     * Probe function.  Otherwise, create an entry ourselves.
     */
    if ((DevType = TypeGetByType(DevInfo->Type)) && DevType->Probe) {
	DevDef.Name 		= DevInfo->Name;
	DevDef.Model 		= DevInfo->Model;
	DevDef.Type 		= DevInfo->Type;
	if (!ProbeData->DevName)
	    ProbeData->DevName	= DevName;
	ProbeData->DevDefine	= &DevDef;
	if (!ProbeData->UseDevInfo)
	    ProbeData->UseDevInfo = DevInfo;
	return((*DevType->Probe)(ProbeData));
    } else {
	if (!DevInfo->Name)
	    DevInfo->Name = DevName;
	if (!DevInfo->Master)
	    DevInfo->Master = MkMasterFromDevData(DevData);
	if (DevData) {
	    if (DevInfo->Unit < 0)
		DevInfo->Unit = DevData->DevUnit;
	    if (DevInfo->NodeID == 0 || DevInfo->NodeID == -1 ||
		DevInfo->NodeID == -2)
		DevInfo->NodeID = DevData->NodeID;
	    if (!DevInfo->Driver)
		DevInfo->Driver = DevData->DevName;
	}
	return(ProbeData->RetDevInfo = DevInfo);
    }
}

/*
 * Search for and call an appropriate probe function for this 
 * device
 */
extern DevInfo_t *ProbeDevice(DevData, TreePtr, Search, OfferDevDef)
    DevData_t 		       *DevData;
    DevInfo_t 		      **TreePtr;
    char		      **Search;
    DevDefine_t		       *OfferDevDef;
{
    register DevDefine_t       *DevDefine = NULL;
    int				DevTypeNum = 0;
    DevType_t		       *DevType = NULL;
    register char 	       *Name = NULL;
    register char	      **cpp;
    static DevFind_t		Find;
    static ProbeData_t		ProbeData;
    DevInfo_t 		       *ProbeUnknown();
    DevInfo_t		     *(*ProbeFunc)() = NULL;
    char		       *DevName = NULL;

    /* Set DevName (no unit) e.g. 'vx' */
    DevName = DevData->DevName;

    /* Get the Device Definetion to use (if any) */
    if (OfferDevDef)
	DevDefine = OfferDevDef;
    else if (DevData->DevName)
	DevDefine = DevDefGet(DevData->DevName, 0, 0);

    /*
     * See if we can find DevType and lookup an appropriate probe function
     */
    if (DevData->DevType)
	DevTypeNum = DevData->DevType;
    else if (!DevDefine && DevData->OSDevInfo && DevData->OSDevInfo->Type)
	DevTypeNum = DevData->OSDevInfo->Type;
    if (DevTypeNum && (DevType = TypeGetByType(DevTypeNum)))
	ProbeFunc = DevType->Probe;

    /*
     * Try to assign a Probe Function if we didn't manage to set one 
     * from DevType above.
     */
    if (!ProbeFunc && DevDefine) {
	if ((*DevDefine->Probe) == NULL)
	    ProbeFunc = DeviceCreate;
	else
	    ProbeFunc = DevDefine->Probe;
    }

    (void) memset(&ProbeData, CNULL, sizeof(ProbeData));
    ProbeData.DevName = DevName;
    ProbeData.DevData = DevData;
    ProbeData.DevDefine = DevDefine;
    ProbeData.DevFile = DevData->DevFile;

    if (ProbeFunc) {
	if (DevDefine) {
	    /*
	     * Add a list of possible device aliases from the DevDefine entry
	     */
	    for (cpp = DevDefine->Aliases; cpp && *cpp; ++cpp) {
		Name = MkDevName(*cpp, DevData->DevUnit, DevDefine->Type, 
				 DevDefine->Flags | DDT_ISALIAS);
		StrArrayAdd(&(ProbeData.AliasNames), Name);
	    }

	    /*
	     * Use the canonical device name whenever possible
	     */
	    if (DevDefine->Name) {
		/* Set the current (passed) name as an alias */
		Name = MkDevName(DevName, DevData->DevUnit,
				 DevDefine->Type, DevDefine->Flags);
		StrArrayAdd(&(ProbeData.AliasNames), Name);
		DevName = DevDefine->Name;
	    }

	    Name = MkDevName(DevName, DevData->DevUnit,
			     DevDefine->Type, DevDefine->Flags);
	} else if (DevType) {
	    Name = MkDevName(DevName, DevData->DevUnit,
			     DevType->Type, 0);
	} else {
	    Name = MkDevName(DevName, DevData->DevUnit, 0, 0);
	}

	DevFindInit(&Find);
	Find.Tree = *TreePtr;
	Find.NodeName = Name;
	if (DevFind(&Find)) {
	    SImsg(SIM_DBG, "ProbeDevice: <%s> already exists.", Name);
	    return((DevInfo_t *) NULL);
	}
	ProbeData.DevName = Name;
	return((*ProbeFunc)(&ProbeData));
    } else if (DevData->OSDevInfo) {
	/*
	 * No probe function, but we do have OSDevInfo which was
	 * passed in by the OS specific DevData gather function, so use it.
	 */
	return(ProbeOSDevInfo(&ProbeData));
    } else if (FLAGS_ON(DevData->Flags, DD_IS_ROOT))
	/*
	 * No probe function and no OS provided DevInfo, so do a generic
	 * probe.
	 */
	return(DeviceCreate(&ProbeData));

    /*
     * The device is unknown to us.  If it's definetly alive,
     * return a minimal device entry for it.  If it's not alive,
     * ignore it.
     */
    if (DoPrintUnknown && DevName && 
	FLAGS_ON(DevData->Flags, DD_IS_ALIVE))
	return(ProbeUnknown(&ProbeData));

    SImsg(SIM_DBG, "ProbeDevice: <%s> is not defined.", ARG(DevName));

    return((DevInfo_t *) NULL);
}

/*
 * Make a master device from a DevData controller
 */
extern DevInfo_t *MkMasterFromDevData(DevData)
    DevData_t 		       *DevData;
{
    register DevInfo_t 	       *DevInfo = NULL;
    register DevDefine_t       *DevDefine;
    int 			type = 0;
    int 			flags = 0;

    if (DevData->CtlrName) {
	DevInfo = NewDevInfo(NULL);
	if (DevDefine = DevDefGet(DevData->CtlrName, 0, 0)) {
	    type = DevDefine->Type;
	    flags = DevDefine->Flags;
	    DevInfo->Model = DevDefine->Model;
	    DevInfo->ModelDesc = DevDefine->Desc;
	}
	DevInfo->Name = MkDevName(DevData->CtlrName,
				    DevData->CtlrUnit, 
				    type, flags);
    }

    return(DevInfo);
}

/*
 * Make the file name of the raw device
 */
extern char *GetRawFile(Name, Part)
    char 		       *Name;
    char 		       *Part;
{
    static char 		rfile[128];

    if (!Name)
	return((char *) NULL);

    (void) snprintf(rfile, sizeof(rfile), "/dev/r%s%s", 
		    Name, (Part) ? Part : "");

    return(rfile);
}

/*
 * Make the file name of the character device
 */
extern char *GetCharFile(Name, Part)
    char 		       *Name;
    char 		       *Part;
{
    static char 		file[128];

    if (!Name)
	return((char *) NULL);

    (void) snprintf(file, sizeof(file), "/dev/%s%s", Name, (Part) ? Part : "");

    return(file);
}

/*
 * Assign a device a new (unique to Device's type/name) Unit number.
 */
static int AssignDevUnit(DevName)
     char		       *DevName;
{
    register struct DevUnits   *Ptr;
    register struct DevUnits   *Last = NULL;
    struct DevUnits	       *New;

    for (Ptr = DevUnits; Ptr; Ptr = Ptr->Next) {
	Last = Ptr;
	if (EQ(Ptr->Name, DevName))
	    break;
    }
    if (!Ptr) {
	Ptr = New = (struct DevUnits *) xcalloc(1, sizeof(struct DevUnits));
	New->Name = strdup(DevName);
	New->LastUnit = -1;
	if (Last)
	    Last->Next = New;
	else
	    DevUnits = New;
    }

    return(++Ptr->LastUnit);
}

/*
 * Make device name
 */
extern char *MkDevName(Name, Unit, Type, DdtFlags)
    char 		       *Name;
    int 			Unit;
    int 			Type;
    int 			DdtFlags;
{
    static char			Buff[128];

    if (!Name)
	return (char *) NULL;

    /* 
     * If DDT_ISALIAS is set, our calling function is doing aliases so
     * we don't want to create a new, unique name.
     */
    if (FLAGS_ON(DdtFlags, DDT_ASSUNIT) && FLAGS_ON(DdtFlags, DDT_ISALIAS))
        return((char *) NULL);

    if (FLAGS_ON(DdtFlags, DDT_ASSUNIT) && Unit < 0) {
	Unit = AssignDevUnit(Name);
    }

    /*
     * Don't attach unit number if this is a pseudo device and DDT_UNITNUM
     * is not set.
     */
    if (Unit < 0 || FLAGS_ON(DdtFlags, DDT_NOUNIT) ||
	((Type == DT_PSEUDO || Type == DT_NONE) && FLAGS_OFF(DdtFlags, 
							     DDT_UNITNUM)))
	snprintf(Buff, sizeof(Buff), "%s", Name);
    else {
	/*
	 * Special handling if the last char of Name is a digit.
	 */
	if (isdigit(Name[strlen(Name) - 1]))
	    snprintf(Buff, sizeof(Buff), "%s/%d", Name, Unit);
	else
	    snprintf(Buff, sizeof(Buff), "%s%d", Name, Unit);
    }

    return(strdup(Buff));
}

/*
 * Make master device name
 */
extern char *MkMasterName(DevData, DdtFlags)
     DevData_t		       *DevData;
     int			DdtFlags;
{
    DevDefine_t		       *DevDef;

    if (!DevData || !DevData->CtlrName)
	return((char *) NULL);

    DevDef = DevDefGet(DevData->CtlrName, 0, 0);

    return(MkDevName(DevData->CtlrName, DevData->CtlrUnit, 
		     (DevDef) ? DevDef->Type : 0,
		     (DevDef) ? DevDef->Flags : 0));
}

/*
 * Create a minimal device type for an unknown device.
 */
extern DevInfo_t *ProbeUnknown(ProbeData)
     ProbeData_t	       *ProbeData;
{
    DevInfo_t		       *DevInfo;
    DevData_t 		       *DevData;

    if (!ProbeData)
	return((DevInfo_t *) NULL);
    DevData = ProbeData->DevData;

    DevInfo = NewDevInfo((DevInfo_t *) NULL);
    DevInfo->Name = strdup(MkDevName(DevData->DevName, 
				       DevData->DevUnit,
				       -1, 0));
    DevInfo->Type = DT_GENERIC;
    DevInfo->Unit = DevData->DevUnit;
    DevInfo->NodeID = DevData->NodeID;
    AddDevDesc(DevInfo, "unknown device type", NULL, DA_APPEND);
    DevInfo->Master = MkMasterFromDevData(DevData);

    return(DevInfo);
}

/*
 * Make a nice looking frequency string.
 */
extern char *FreqStr(freq)
    u_long			freq;
{
    static char			buff[64];

    if (freq > MHERTZ)
	(void) snprintf(buff, sizeof(buff), "%ld MHz", freq / MHERTZ);
    else
	(void) snprintf(buff, sizeof(buff), "%ld Hz", freq);

    return(buff);
}
