/*
 * Copyright (c) 1992-2002 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/4.2/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.2 $";
#endif

#include "defs.h"
#include "dmi.h"

/*
 * Memory Array Locations
 * See section 3.3.17.1
 */
static char *Locations[] = {
    "",
    "",
    "",
    "System Board",
    "ISA add-on card",
    "EISA add-on card",
    "PCI add-on card",
    "MCA add-on card",
    "PCMCIA add-on card",
    "Proprietary add-on card",
    "NuBus add-on card",
    "PC-98/C20 add-on card",
    "PC-98/C24 add-on card",
    "PC-98/E add-on card",
    "PC-98/Local-bus add-on card",
    NULL
};

/*
 * Usage
 * See section 3.3.17.2
 */
static char *Usages[] = {
    "",
    "",
    "",
    "System memory",
    "Video memory",
    "Flash memory",
    "Non-volatile RAM",
    "Cache memory",
    NULL
};

/* 
 * Error Correction Types
 * See section 3.3.17.3
 */
static char *CorrectionTypes[] = {
    "",
    "",
    "",
    "None",
    "Parity",
    "Single-bit ECC",
    "Multi-bit ECC",
    "CRC",
    NULL
};

/*
 * Decode DMI Memory Array (DMI_STRUCT_MEMARRAY)
 */
extern DevInfo_t *DmiDecodeMemarray(Hdr, Data, Dmi, Query)
     DmiHeader_t	       *Hdr;
     void		       *Data;
     Dmi_t		       *Dmi;
     DmiQuery_t		       *Query;
{
    DevInfo_t		       *DevInfo = NULL;
    DevDesc_t		      **lp;
    DmiStructMemarray_t	       *Mem;
    char		       *cp;
    static char			Buff[128];
    static int			Unit;
    register  int		i;

    Mem = (DmiStructMemarray_t *) Data;

    SImsg(SIM_DBG, 
	  "DMI MEMARRAY: Handle=0x%04X Capacity=%dKB Members=%d",
	  Hdr->Handle, Mem->Capacity, Mem->Count);
    SImsg(SIM_DBG, 
	  "DMI MEMARRAY:\tsizeof(DmiStructMemarray_t)=%d (0x%X) Hdr->Length=%d (0x%X)",
	  sizeof(DmiStructMemarray_t), sizeof(DmiStructMemarray_t), 
	  Hdr->Length, Hdr->Length);

    /*
     * If this is a specific query, then set the values now and return
     */
    if (Query && Query->Type == Hdr->Type) {
	/* Return nothing for now */
	return DevInfo;
    }

    DevInfo = NewDevInfo(NULL);
    DevInfo->Type = DT_MEMCTL;
    lp = &(DevInfo->DescList);
    DevInfo->Unit = Unit++;
    snprintf(Buff, sizeof(Buff), "memarray%d", DevInfo->Unit);
    DevInfo->Name = strdup(Buff);

    AddDesc(lp, "Max Capacity", GetSizeStr((Large_t)Mem->Capacity, KBYTES), 0);

    if (cp = GetArrayEntry(Locations, Mem->Location))
	AddDesc(lp, "Location", cp, 0);
    if (cp = GetArrayEntry(Usages, Mem->Use))
	AddDesc(lp, "Usage", cp, 0);
    if (cp = GetArrayEntry(CorrectionTypes, Mem->ErrCorrect))
	AddDesc(lp, "Error Correction", cp, 0);

    return DevInfo;
}
