#
# $Id: uiIcon.pm,v 1.4 2002/03/15 21:50:13 mcooper Exp $
#
# UI Icon functions
#

package uiIcon;

require 5.003;

require Exporter;

use mcDebug;
use uiMisc;
use uiGlobals;

#
# Map of types to icon images
#
$Icons = {
	     # Class types
	     'general' 			=> 'tower',
	     'software' 		=> 'software',
	     'kernel' 			=> 'fleur',
	     'sysconf' 			=> 'scroll',
	     'hardware' 		=> 'hardware',
	     'partition' 		=> 'disk',
	     'printer'	 		=> 'printer',
	     # Device types
	     'pseudo device' 		=> 'program',
	     'device driver' 		=> 'program',
	     'system bus' 		=> 'bus',
	     'bus bridge' 		=> 'bridge',
	     'controller' 		=> 'card',
	     'system card' 		=> 'card',
	     'disk controller' 		=> 'card',
	     'frame buffer' 		=> 'card',
	     'graphics accelerator' 	=> 'card',
	     'keyboard' 		=> 'keyboard',
	     'mouse' 			=> 'mouse',
	     'pointer' 			=> 'mouse',
	     'tape controller' 		=> 'card',
	     'tape drive' 		=> 'tape',
	     'cache' 			=> 'memory',
	     'memory' 			=> 'memory',
	     'system bios'		=> 'memory',
	     'system board'		=> 'board',
	     'usb' 			=> 'usb',
	     'serial device' 		=> 'serial',
	     'parallel device' 		=> 'parallel',
	     'audio device' 		=> 'audio',
	     'multi function card' 	=> 'hardware',
	     'video monitor' 		=> 'monitor',
	     'printer' 			=> 'printer3',
	     'worm' 			=> 'hardware',
	     'scanner' 			=> 'scanner',
	     'console' 			=> 'monitor',
	     'disk drive' 		=> 'disk',
	     'cd drive' 		=> 'cdrom',
	     'cd-rom drive' 		=> 'cdrom',
	     'dvd drive' 		=> 'cdrom',
	     'cpu' 			=> 'cpu',
	     'processor' 		=> 'cpu',
	     'floppy disk drive' 	=> 'floppy',
	     'floppy disk controller' 	=> 'card',
	     'network interface' 	=> 'network',
	     'joy stick' 		=> 'joystick',
	     'enclosure' 		=> 'tower',
	     'unknownhardware' 		=> 'hardware',
};

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   GetIcon
		   $Icons
		  );
}

#
# Retrieve the name of an appropriate icon
#
sub GetIcon {
    my $Args = shift;
    my $Entry = $Args->{Entry};
    my $Path = $Args->{Path};
    my $Class = $Args->{Class};
    my $Icon;

    if ($Entry =~ /mcDevInfo/) {
	if ($Entry->ClassType =~ /USB/ ||
	    $Entry->Model =~ /USB/ ||
	    $Entry->ModelDesc =~ /USB/) {
	    $Icon = $Icons->{'usb'};
	}
	if (!$Icon) {
	    my $Name = lc($Entry->Type);
	    $Icon = $Icons->{$Name};
	}
	if (!$Icon) {
	    $Icon = 'card';
	}
    }
    if (!$Icon && $Entry =~ /PartInfo/) {
	$Icon = $Icons->{'disk'};
    }
    if (!$Icon && $Entry =~ /CacheInfo/) {
	$Icon = $Icons->{'cache'};
    }
    if (!$Icon) {
	$Icon = $Icons->{lc($Path)};
    }
    if (!$Icon) {
	$Icon = $Icons->{lc($Entry)};
    }
    if (!$Icon) {
	# Try looking for "${class}unknown"
	my $n = sprintf("unknown%s", lc($Class));
	$Icon = $Icons->{$n};
    }

    return $Icon;
}

1;
