#
# $Id: uiPath.pm,v 1.5 2002/03/09 01:09:29 mcooper Exp $
#
# UI Path functions
#

package uiPath;

require 5.003;

require Exporter;

use mcDebug;
use uiMisc;
use uiGlobals;
use uiArrange;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   PathTable PathExists PathCount
		   PathGet PathGetByType
		  );
}

#
# Given a Path, return the Table Path exists in or should be placed in.
# Path does not necessarily have to exit in Table.
#
sub PathTable {
    my $Path = shift;

    if (my $Class = uiClass::ClassGet($Path)) {
	return uiClass::ClassTable({Class=>$Class});
    }
}

#
# Validate that $path is a valid Tree path
#
sub PathExists {
    my $Path = shift;

    if (my $Table = PathTable($Path)) {
	if ($Table->{$Path}) {
	    return 1;
	}
    }

    return 0;
}

sub PathCount {
    my $Path = shift;

    my @p = split($PathSep, $Path);

    return $#p + 1;
}

#
# Get a Tree Path using the device type.
# Applies only to Hardware.
#
sub PathGetByType {
    my $Args = shift;
    my $Obj = $Args->{Obj};
    my $Name = $Args->{Name};
    my $Class = $Args->{Class};
    my $TypePtr = $Args->{TypePtr};

    if (!$Obj || !$Name || !$Class) {
	return;
    }

    if (!Eq($Class->{Name}, 'Hardware')) {
	Debug("PathGetByType: Class <%s> is not supported by this func.",
	      $Class->{Name});
	return;
    }

    my $Entry = $Obj->{$Name};
    my $Type = $Entry->Type;
    my $type = Casify($Type);
    if (!length($Type)) {
	$type = 'Unknown';
    }

    my $Path = sprintf("%s%s%s",
		       $Class->{Name}, $PathSep, $type);

    if ($TypePtr) {
	$$TypePtr = $type;
    }

    return $Path;
}

#
# Get a Path name
# -RECURSE-
#
sub PathGet {
    my $Args = shift;
    my $Obj = $Args->{Obj};
    my $Entry = $Args->{Entry};
    my $Name = $Args->{Name};
    my $Top = $Args->{Top};
    my $MasterName = $Args->{MasterName};
    my $Path;
    my $Master;

    if ($MasterName && !$Entry) {
	$Entry = $Obj->{$MasterName};
    }

    if (!$Entry) {
	return;
    }

    if ($Entry =~ /HASH/) {
	if (HasMember($Entry, 'Name') && $Entry->Name) {
	    $Name = $Entry->Name;
	}
	#
	# Special case for Hardware arrangement by Type
	#
	if ($Entry =~ /DevInfo/) {
	    my $class = uiClass::ClassGet($CL_HARDWARE);
	    if (ArrangeIs({Name=>$AR_TYPE, Class=>$class})) {
		$Path = PathGetByType({Name=>$Name,
				       Class=>$class,
				       Obj=>$Obj});
		#
		# Now create the path for the device itself.
		#
		$Path = sprintf("%s%s%s", $Path, $PathSep, $Name);
	    }
	}
	#
	# Arrange by Connection
	#
	if (!$Path && HasMember($Entry, 'Master') && $Entry->Master) {
	    my $Parent = PathGet({MasterName=>$Entry->Master, Obj=>$Obj,
				      Top=>$Top});
	    if ($Parent) {
		$Path = sprintf("%s%s%s", $Parent, $PathSep, $Name);
	    }
	}
    }
    if (!$Path) {
	if ($Top) {
	    $Path = $Top . $PathSep . $Name;
	} else {
	    $Path = $Name;
	}
    }

##    Debug("PathGet node=<%s> path=<%s>",
##	  $Name, $Path);

    return $Path;
}

1;
