/*
 * Copyright (c) 1992-2002 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/4.2/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.9 $";
#endif

/*
 * Columns functions
 */

#include "defs.h"
#include "html.h"
#if	defined(HAVE_STDLIB_H)
#include <stdlib.h>
#endif

/*
 * Take an array of percentages and return a format string which uses no
 * more than the terminal width.
 */
extern char **ColGetFmt(Header)
     HeaderFormat_t	       *Header;
{
    register int		i;
    int				w;
    int				Min;
    int				Used = 0;
    int				EnforceMin;
    static int			MaxWidth;
    static char			Fmt[32];
    static char		      **Argv;

    if (!MaxWidth)
	if (MaxWidth = TermWidth())
	    /* Avoid column wrap problems by not using the last column */
	    MaxWidth--;

    /*
     * Allocate Argv and insure last slot is NULL
     */
    Argv = (char **) calloc((size_t)(Header->PercentsCount + 1),
			    sizeof(char *));
    if (!Argv)
	return (char **) NULL;

    /*
     * Check to see if we're going to enforce minimums.
     * If any member of Minimums is == 0, then we don't enforce.
     */
    EnforceMin = 1;
    for (i = 0; i < Header->MinimumsCount; ++i)
	if (Header->Minimums[i] == 0) {
	    EnforceMin = 0;
	    break;
	}

    /*
     * Build up Argv
     */
    for (i = 0; i < Header->PercentsCount; ++i) {
	if (i < Header->MinimumsCount) {
	    if (Header->MinOvers[i]) {
		/* Use the override value */
		Min = Header->MinOvers[i];
	    } else {
		/* Use the default value */
		Min = Header->Minimums[i];
	    }
	} else {
	    Min = 0;
	}

	if (Header->Percents[i]) {
	    /* Calculate width based on percentage */
	    w = (int) (((float)Header->Percents[i] / (float)100) * 
		       (float) MaxWidth);
	    /* Deduct 2 for rounding and 1 for space between args */
	    w -= 3;
	    /* Insure Width is not less than the minimum */
	    if (w < Min)
		w = Min;
	} else {
	    w = Min;
	}

	/*
	 * Insure we don't exceed MaxWidth, but only if we're enforcing
	 * minimums.
	 */
	if (EnforceMin && (Used + w >= MaxWidth)) {
	    w = MaxWidth - Used;
	    if (w < 0)
		continue;
	}
	/* Width of this fmt + SPACE */
	Used += w + 1;

	if (w == 0)
	    snprintf(Fmt, sizeof(Fmt), "%%s");
	else
	    snprintf(Fmt, sizeof(Fmt), "%%-%d.%ds", w, w);

	Argv[i] = strdup(Fmt);
    }

    return Argv;
}

/*
 * Given Format (from ColGetFmt()) return a single format string.
 */
extern char *ColGetFmtStr(Format)
     char		      **Format;
{
    register int		i;
    char			FmtStr[1024];

    if (!Format)
	return (char *) NULL;

    FmtStr[0] = CNULL;
    for (i = 0; Format[i]; ++i) {
	if (i > 0)
	    StrAppend(FmtStr, sizeof(FmtStr), " ");
	StrAppend(FmtStr, sizeof(FmtStr), Format[i]);
    }

    if (FmtStr[0]) {
	StrAppend(FmtStr, sizeof(FmtStr), "\n");
	return strdup(FmtStr);
    }

    return (char *) NULL;
}

/*
 * Show a Column Value in OE_TEXT
 */
extern void ColShowValueText(Class, Lbl, Key, Value)
     ClassInfo_t	       *Class;
     char 		       *Lbl;
     char 		       *Key;
     char 		       *Value;
{
    int				Len;
    static char		       *FormatStr;
    static char		       *FormatClass;
    char		      **Format;

    if (!Value || !*Value)
	return;

    /*
     * Validate that the FormatStr was made for the specified Class.
     * If not, invalidate it to force a new one to be generated.
     */
    if (FormatStr && FormatClass && !EQ(Class->Name, FormatClass))
	FormatStr = NULL;
    FormatClass = Class->Name;

    if (!FormatStr) {
	Class = ClassGetByName(Class->Name);
	if (!Class) {
	    SImsg(SIM_DBG, "ColShowValueText: Cannot find %s class",
		  Class->Name);
	    return;
	}

	if (VL_ALL)
	    Format = ColGetFmt(Class->HeadersAll);
	else
	    Format = ColGetFmt(Class->Headers);
	if (!Format)
	    return;

	FormatStr = ColGetFmtStr(Format);
	if (!FormatStr)
	    return;
    }

    if (VL_ALL)
	SImsg(SIM_INFO, FormatStr,
	      CPRT(Lbl), CPRT(Key), CPRT(Value));
    else
	SImsg(SIM_INFO, FormatStr,
	      CPRT(Lbl), CPRT(Value));
}

/*
 * Show a Column Value in OE_HTML
 */
extern void ColShowValueHtml(Class, Lbl, Key, Value)
     ClassInfo_t	       *Class;
    char 		       *Lbl;
    char 		       *Key;
    char 		       *Value;
{
    int				Len;
    char		       *DataArgv[4];
    static char		       *FormatArgv[4];
    int				DataArgc = 0;
    static int			Counter;
    char		        Bgcolor[128];

    if (!Value || !*Value)
	return;

    if (VL_ALL) {
	DataArgv[DataArgc++] = CPRT(Lbl); 
	DataArgv[DataArgc++] = CPRT(Key); 
	DataArgv[DataArgc++] = CPRT(Value);
	DataArgv[DataArgc] = NULL;
    } else {
	DataArgv[DataArgc++] = CPRT(Lbl); 
	DataArgv[DataArgc++] = CPRT(Value);
	DataArgv[DataArgc] = NULL;
    }

    /* Rows alternate colors */
    if (Counter++ % 2)
	snprintf(Bgcolor, sizeof(Bgcolor), "bgcolor=\"%s\"",
		 HTML_BGLINE_1A);
    else
	snprintf(Bgcolor, sizeof(Bgcolor), "bgcolor=\"%s\"",
		 HTML_BGLINE_1B);

    FormatArgv[0] = "NOWRAP";	/* Don't wrap first field (Lbl) */

    HtmlTableData(DataArgv, DataArgc, FormatArgv, Bgcolor);
}

/*
 * Show a Column Value
 */
extern void ColShowValue(Class, Lbl, Key, Value, MaxLen)
     ClassInfo_t	       *Class;
     char 		       *Lbl;
     char 		       *Key;
     char 		       *Value;
     int			MaxLen;
{
    switch (EncodeType) {
    case OE_TEXT:
	ColShowValueText(Class, Lbl, Key, Value);
	break;
    case OE_HTML:
	ColShowValueHtml(Class, Lbl, Key, Value);
	break;
    }
}

/*
 * Display column headers as OE_TEXT
 */
extern void ColHeaderText(Header)
     HeaderFormat_t	       *Header;
{
    char		      **Format;
    register int		i;

    Format = ColGetFmt(Header);
    if (!Format)
	return;

    for (i = 0; i < Header->PercentsCount; ++i) {
	if (i > 0)
	    SImsg(SIM_INFO|SIM_NOLBL, " ");
	SImsg(SIM_INFO|SIM_NOLBL, Format[i], Header->Headers[i]);
	(void) free(Format[i]);
    }
    (void) free(Format);

    SImsg(SIM_INFO|SIM_NOLBL, "\n");
}
