/*
 * Copyright (c) 1992-2002 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/4.2/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.17 $";
#endif

/*
 * Show general functions
 */

#include "defs.h"
#include "html.h"
#if HAVE_STDLIB_H
# include <stdlib.h>
#endif

/*
 * Show Off Set space
 */
extern void ShowOffSet(cnt)
    int 			cnt;
{
    float			f;

    if (FormatType == FT_TREE)
	SImsg(SIM_INFO, "%*s", cnt, "");
}

/*
 * Start of a ShowLabel block
 */
extern void ShowLabelStart()
{
    switch (EncodeType) {
    case OE_HTML:
	HtmlShowLabelStart();
	break;
    }
}

/*
 * End of a ShowLabel block
 */
extern void ShowLabelEnd()
{
    switch (EncodeType) {
    case OE_HTML:
	HtmlShowLabelEnd();
	break;
    }
}

/*
 * Show a label
 */
extern void ShowLabel(Label, Value, OffSet)
    char 		       *Label;
    char 		       *Value;
    int 			OffSet;
{
    switch (FormatType) {
    case FT_COLUMNS:
    case FT_TREE:
	switch (EncodeType) {
	case OE_TEXT:
	    ShowOffSet(OffSet);
	    if (VL_CONFIG)
		SImsg(SIM_INFO, "%*s%-22s: %s\n", 
		      OffSetAmt, "", PRTS(Label), PRTS(Value));
	    else
		SImsg(SIM_INFO, "%*s%22s: %s\n",
		      OffSetAmt, "", PRTS(Label), PRTS(Value));
	    break;
	case OE_HTML:
	    HtmlShowLabel(Label, Value, OffSet/OffSetAmt);
	    break;
	}
	break;
    }
}

/*
 * Return a possibly encoded version of Name
 */
extern char *ShowEncodeName(Name)
     char		       *Name;
{
    switch (EncodeType) {
    case OE_HTML:
	return HtmlEncodeName(Name);
    default:
	return Name;
    }
}

/*
 * Show a single line
 */
extern void ShowLine(OffSet, String)
     int			OffSet;
     char		       *String;
{
    switch (FormatType) {
    case FT_TREE:
	switch (EncodeType) {
	case OE_TEXT:
	    TreeLine(OffSet, String);
	    break;
	case OE_HTML:
	    HtmlLine(OffSet, String);
	    break;
	}
	break;
    }
}

/*
 * End any previous calls to ShowLine()
 */
extern void ShowLineEnd()
{
    switch (EncodeType) {
    case OE_HTML:
	HtmlLineEnd();
	break;
    }
}

/*
 * Show Key Value as FT_TREE + OE_TEXT
 */
extern void ShowKeyValueTreeText(Lbl, Key, Value, Max)
    char 		       *Lbl;
    char 		       *Key;
    char 		       *Value;
    int			       *Max;
{
    char			Buff[256];
    int				Len;

    if (!Value || !*Value)
	return;

    if (VL_TERSE) {
	SImsg(SIM_INFO, "%s", Value);
    } else if (VL_BRIEF) {
	SImsg(SIM_INFO, "%s is %s", Key, Value);
    } else {
	if (Lbl) {
	    (void) strcpy(Buff, Lbl);
	    if (VL_ALL) {
		Len = strlen(Lbl);
		(void) snprintf(Buff + Len, sizeof(Buff) - Len, " (%s)", Key);
	    }
	} else
	    (void) strcpy(Buff, Key);
	(void) strcat(Buff, " is ");
	Len = Max[0] + ((Max[1] > 0) ? Max[1] : 0) + 4;
	SImsg(SIM_INFO, "%-*s %s", Len, Buff, Value);
    }

    if (Value[strlen(Value) - 1] != '\n')
	SImsg(SIM_INFO, "\n");
}

/*
 * Show Key/Value info in FT_TREE
 */
extern void ShowKeyValueTree(Label, Key, Value, Max)
     char		       *Label;
     char		       *Key;
     char		       *Value;
     int		       *Max;
{
    if (!Value)
	Value = UnSupported;

    switch (EncodeType) {
    case OE_TEXT:
	ShowKeyValueTreeText(Label, Key, Value, Max);
	break;
    case OE_HTML:
	HtmlKeyValue(Label, Key, Value);
	break;
    }
}

/*
 * Show a class Banner
 */
extern void ShowBanner(MyClass)
    ClassInfo_t		       *MyClass;
{
    char		       *Banner = NULL;

    Banner = (MyClass->Banner) ? MyClass->Banner : MyClass->Label;

    if (Banner && !VL_TERSE && !VL_BRIEF)
	if (FormatType == FT_TREE || FormatType == FT_COLUMNS)
	    switch (EncodeType) {
	    case OE_TEXT:
		TextBanner(Banner);
		break;
	    case OE_HTML:
		HtmlBanner(Banner);
		break;
	    }
}

/*
 * Start a show for the given Class
 */
extern void ShowStart(Class, MaxSizes)
     ClassInfo_t	       *Class;
     int		       *MaxSizes;
{
    char		       *HtmlAlign = "ALIGN=\"left\"";
    static char		      **FormatArgv;
    static char			TableArgs[128];
    HeaderFormat_t	       *Headers;
    register int		i;

    ShowBanner(Class);

    switch (FormatType) {
    case FT_COLUMNS:
	if (VL_ALL)
	    Headers = Class->HeadersAll;
	else
	    Headers = Class->Headers;

	(void) memset(&Headers->MinOvers, CNULL, 
		      sizeof(Headers->MinOvers)/sizeof(int));

	/* Copy/set the Max string sizes into MinOvers */
	if (MaxSizes)
	    for (i = 0; i < Headers->MinimumsCount && MaxSizes[i] != -1; ++i)
		Headers->MinOvers[i] = MaxSizes[i];

	switch (EncodeType) {
	case OE_TEXT:
	    ColHeaderText(Headers);
	    break;
	case OE_HTML:
	    HtmlTableStart(NULL);	/* ShowEnd() will terminate this */

	    if (FormatArgv)
		(void) free(FormatArgv);
	    FormatArgv = (char **) calloc(Headers->HeadersCount + 1,
					  sizeof(char *));
	    if (!FormatArgv)
		MemFatal("ShowStart() calloc");

	    for (i = 0; i < Headers->HeadersCount; ++i)
		FormatArgv[i] = HtmlAlign;

	    snprintf(TableArgs, sizeof(TableArgs), "bgcolor=\"%s\"",
		     HTML_BGHEAD_1);
	    HtmlTableHeader(Headers->Headers, Headers->HeadersCount,
			    FormatArgv, TableArgs);
	    break;
	}
	break;
    }

    if (FormatType == FT_TREE && EncodeType == OE_HTML)
	HtmlKeyValueSet();
}

/*
 * End a class show (ShowStart())
 */
extern void ShowEnd(Class)
     ClassInfo_t	       *Class;
{
    switch (FormatType) {
    case FT_COLUMNS:
	switch (EncodeType) {
	case OE_HTML:
	    HtmlTableEnd();	/* Terminate ShowStart() table */
	    break;
	}
	break;
    }

    /*
     * Terminate any pending lines.
     * Need to do this before HtmlKeyValueSet() to insure our </UL> are
     * inside of <TABLE></TABLE>.
     */
    ShowLineEnd();

    if (FormatType == FT_TREE && EncodeType == OE_HTML)
	HtmlKeyValueSet();
}
