/*
 * Copyright (c) 1992-2002 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/4.2/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.13 $";
#endif

/*
 * System Configuration
 *
 * Use the sysconf() routine to show system configuration info.
 * This routine is available on POSIX compliant systems.
 */
#include "defs.h"

/*
 * List valid arguments for the SysConf class.
 */
extern void SysConfList()
{
    register Define_t	       *DefPtr;

    DefPtr = (Define_t *) DefGetList(DL_SYSCONF); 
    if (!DefPtr) {
	SImsg(SIM_WARN, "No sysconf variables are defined.");
	return;
    }

    SImsg(SIM_INFO, "\n\nThe following are valid arguments for `-class SysConf -show Name1,Name2,...':\n\n");
    SImsg(SIM_INFO, "%-25s %s\n", "NAME", "DESCRIPTION");

    for ( ; DefPtr; DefPtr = DefPtr->Next)
	if (sysconf(DefPtr->KeyNum) > 0)
	    SImsg(SIM_INFO, "%-25s %s\n", DefPtr->KeyStr, DefPtr->ValStr2);
}

/*
 * Check to see if a symbol named String appears
 * in Argv.
 * Return 1 if found.
 * Return 0 if not found.
 */
static int HasName(String, Argv)
    char		       *String;
    char		      **Argv;
{
    register char	      **cpp;

    for (cpp = Argv; cpp && *cpp; ++cpp)
	if (EQ(String, *cpp))
	    return(1);

    return(0);
}

/*
 * Show System Configuration information utilizing the sysconf()
 * system call.
 */
extern int SysConfShow(MyClass, Names)
    ClassInfo_t		       *MyClass;
    char		      **Names;
{
    static char		       *RptData[3];
    int				Max[3];
    register int		Len;
    Define_t		       *SysConfDef;
    register Define_t	       *DefPtr;
    static MCSIquery_t		Query;

    (void) memset(&Query, CNULL, sizeof(Query));
    Query.Op = MCSIOP_CREATE;
    Query.Cmd = MCSI_SYSCONF;
    if (mcSysInfo(&Query) != 0)
	return -1;

    SysConfDef = (Define_t *) Query.Out;
    if (!SysConfDef) {
	SImsg(SIM_WARN, "No sysconf variables are available.");
	return CLASS_UNSUPPORTED;
    }

    /*
     * Calculate the longest string values
     */
    Max[0] = Max[1] = Max[2] = -1;
    for (DefPtr = SysConfDef; DefPtr; DefPtr = DefPtr->Next) {
	if (Names && !HasName(DefPtr->KeyStr, Names))
	    continue;
	if (!DefPtr->ValStr3 && !DefPtr->ValInt1)
	    /* No value retrieved */
	    continue;

	/* Description */
	Len = strlen(DefPtr->ValStr2);
	if (Len > Max[0])
	    Max[0] = Len;

	if (VL_ALL) {
	    /* Key/Name */
	    Len = strlen(DefPtr->KeyStr);
	    if (Len > Max[1])
		Max[1] = Len;
	}
    }

    ShowStart(MyClass, Max);

    for (DefPtr = SysConfDef; DefPtr; DefPtr = DefPtr->Next) {
	if (Names && !HasName(DefPtr->KeyStr, Names))
	    continue;

	/* Skip entries that are not available */
	if (DefPtr->ValInt1 < 0)
	    continue;

	switch (FormatType) {
	case FT_TREE:
	    ShowKeyValueTree(DefPtr->ValStr2, DefPtr->KeyStr,
			     DefPtr->ValStr3, Max);
	    break;
	case FT_COLUMNS:
	    ColShowValue(MyClass,
			 DefPtr->ValStr2, DefPtr->KeyStr, DefPtr->ValStr3);
	    break;
	case FT_REPORT:
	    RptData[0] = DefPtr->ValStr2;
	    RptData[1] = DefPtr->ValStr3;
	    Report(CN_SYSCONF, NULL, DefPtr->KeyStr, RptData, 2);
	    break;
	}
    }

    ShowEnd(MyClass);

    return 0;
}
